@@ -41,7 +41,6 @@ class QgsPathResolver;
  * \class QgsSimpleMarkerSymbolLayerBase
  * \brief Abstract base class for simple marker symbol layers. Handles creation of the symbol shapes but
  * leaves the actual drawing of the symbols to subclasses.
- * \since QGIS 2.16
  */
 class CORE_EXPORT QgsSimpleMarkerSymbolLayerBase : public QgsMarkerSymbolLayer
 {
@@ -216,6 +215,7 @@ class CORE_EXPORT QgsSimpleMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
     // reimplemented from base classes
 
     QString layerType() const override;
+    Qgis::SymbolLayerFlags flags() const override;
     void startRender( QgsSymbolRenderContext &context ) override;
     void renderPoint( QPointF point, QgsSymbolRenderContext &context ) override;
     QVariantMap properties() const override;
@@ -258,7 +258,6 @@ class CORE_EXPORT QgsSimpleMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
      * \see setStrokeStyle()
      * \see strokeColor()
      * \see penJoinStyle()
-     * \since QGIS 2.4
     */
     Qt::PenStyle strokeStyle() const { return mStrokeStyle; }
 
@@ -268,7 +267,6 @@ class CORE_EXPORT QgsSimpleMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
      * \see strokeStyle()
      * \see setStrokeColor()
      * \see setPenJoinStyle()
-     * \since QGIS 2.4
     */
     void setStrokeStyle( Qt::PenStyle strokeStyle ) { mStrokeStyle = strokeStyle; }
 
@@ -278,7 +276,6 @@ class CORE_EXPORT QgsSimpleMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
      * \see penCapStyle()
      * \see strokeColor()
      * \see strokeStyle()
-     * \since QGIS 2.16
     */
     Qt::PenJoinStyle penJoinStyle() const { return mPenJoinStyle; }
 
@@ -289,7 +286,6 @@ class CORE_EXPORT QgsSimpleMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
      * \see setPenCapStyle()
      * \see setStrokeColor()
      * \see setStrokeStyle()
-     * \since QGIS 2.16
     */
     void setPenJoinStyle( Qt::PenJoinStyle style ) { mPenJoinStyle = style; }
 
@@ -430,7 +426,6 @@ class CORE_EXPORT QgsSimpleMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
  * \class QgsFilledMarkerSymbolLayer
  * \brief Filled marker symbol layer, consisting of a shape which is rendered using a QgsFillSymbol. This allows
  * the symbol to support advanced styling of the interior and stroke of the shape.
- * \since QGIS 2.16
  */
 class CORE_EXPORT QgsFilledMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayerBase
 {
@@ -460,6 +455,8 @@ class CORE_EXPORT QgsFilledMarkerSymbolLayer : public QgsSimpleMarkerSymbolLayer
     QString layerType() const override;
     void startRender( QgsSymbolRenderContext &context ) override;
     void stopRender( QgsSymbolRenderContext &context ) override;
+    void startFeatureRender( const QgsFeature &feature, QgsRenderContext &context ) override;
+    void stopFeatureRender( const QgsFeature &feature, QgsRenderContext &context ) override;
     QVariantMap properties() const override;
     QgsFilledMarkerSymbolLayer *clone() const override SIP_FACTORY;
     QgsSymbol *subSymbol() override;
@@ -501,6 +498,7 @@ class CORE_EXPORT QgsSvgMarkerSymbolLayer : public QgsMarkerSymbolLayer
                              double size = DEFAULT_SVGMARKER_SIZE,
                              double angle = DEFAULT_SVGMARKER_ANGLE,
                              Qgis::ScaleMethod scaleMethod = DEFAULT_SCALE_METHOD );
+    QgsSvgMarkerSymbolLayer( const QgsSvgMarkerSymbolLayer &other ) SIP_SKIP;
 
     ~QgsSvgMarkerSymbolLayer() override;
 
@@ -513,25 +511,19 @@ class CORE_EXPORT QgsSvgMarkerSymbolLayer : public QgsMarkerSymbolLayer
     /**
      * Turns relative paths in properties map to absolute when reading and vice versa when writing.
      * Used internally when reading/writing symbols.
-     * \since QGIS 3.0
      */
     static void resolvePaths( QVariantMap &properties, const QgsPathResolver &pathResolver, bool saving );
 
     // implemented from base classes
 
     QString layerType() const override;
-
+    Qgis::SymbolLayerFlags flags() const override;
     void startRender( QgsSymbolRenderContext &context ) override;
-
     void stopRender( QgsSymbolRenderContext &context ) override;
-
     void renderPoint( QPointF point, QgsSymbolRenderContext &context ) override;
-
     QVariantMap properties() const override;
     bool usesMapUnits() const override;
-
     QgsSvgMarkerSymbolLayer *clone() const override SIP_FACTORY;
-
     void writeSldMarker( QDomDocument &doc, QDomElement &element, const QVariantMap &props ) const override;
 
     /**
@@ -577,7 +569,8 @@ class CORE_EXPORT QgsSvgMarkerSymbolLayer : public QgsMarkerSymbolLayer
     /**
      * Returns the marker aspect ratio between width and height to be used in rendering,
      * if the value set is lower or equal to 0 the aspect ratio will be preserved in rendering
-     * \see setFixedAspectRatio() QgsSvgCache
+     * \see setFixedAspectRatio()
+     * \see QgsSvgCache
      */
     double fixedAspectRatio() const { return mFixedAspectRatio; }
 
@@ -585,7 +578,8 @@ class CORE_EXPORT QgsSvgMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * Set the marker aspect ratio between width and height to be used in rendering,
      * if the value set is lower or equal to 0 the aspect ratio will be preserved in rendering
      * \param ratio Fixed Aspect Ratio
-     * \see fixedAspectRatio() QgsSvgCache
+     * \see fixedAspectRatio()
+     * \see QgsSvgCache
      */
     void setFixedAspectRatio( double ratio ) { mFixedAspectRatio = ratio; }
 
@@ -706,14 +700,13 @@ class CORE_EXPORT QgsRasterMarkerSymbolLayer : public QgsMarkerSymbolLayer
     /**
      * Turns relative paths in properties map to absolute when reading and vice versa when writing.
      * Used internally when reading/writing symbols.
-     * \since QGIS 3.0
      */
     static void resolvePaths( QVariantMap &properties, const QgsPathResolver &pathResolver, bool saving );
 
     // implemented from base classes
 
     QString layerType() const override;
-
+    Qgis::SymbolLayerFlags flags() const override;
     void renderPoint( QPointF point, QgsSymbolRenderContext &context ) override;
     QVariantMap properties() const override;
     QgsRasterMarkerSymbolLayer *clone() const override SIP_FACTORY;
@@ -785,7 +778,8 @@ class CORE_EXPORT QgsRasterMarkerSymbolLayer : public QgsMarkerSymbolLayer
     /**
      * Returns the marker aspect ratio between width and height to be used in rendering,
      * if the value set is lower or equal to 0 the aspect ratio will be preserved in rendering
-     * \see setFixedAspectRatio() QgsSvgCache
+     * \see setFixedAspectRatio()
+     * \see QgsSvgCache
      */
     double fixedAspectRatio() const { return mFixedAspectRatio; }
 
@@ -793,7 +787,8 @@ class CORE_EXPORT QgsRasterMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * Set the marker aspect ratio between width and height to be used in rendering,
      * if the value set is lower or equal to 0 the aspect ratio will be preserved in rendering
      * \param ratio Fixed Aspect Ratio
-     * \see fixedAspectRatio() QgsSvgCache
+     * \see fixedAspectRatio()
+     * \see QgsSvgCache
      */
     void setFixedAspectRatio( double ratio ) { mFixedAspectRatio = ratio; }
 
@@ -893,6 +888,7 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
     // implemented from base classes
 
     QString layerType() const override;
+    Qgis::SymbolLayerFlags flags() const override;
 
     void startRender( QgsSymbolRenderContext &context ) override;
 
@@ -964,7 +960,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * \see strokeWidthUnit()
      * \see strokeWidthMapUnitScale()
      *
-     * \since QGIS 2.16
     */
     double strokeWidth() const { return mStrokeWidth; }
 
@@ -975,7 +970,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * \see setStrokeWidthUnit()
      * \see setStrokeWidthMapUnitScale()
      *
-     * \since QGIS 2.16
     */
     void setStrokeWidth( double width ) { mStrokeWidth = width; }
 
@@ -986,7 +980,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * \see strokeWidth()
      * \see strokeWidthMapUnitScale()
      *
-     * \since QGIS 2.16
     */
     Qgis::RenderUnit strokeWidthUnit() const { return mStrokeWidthUnit; }
 
@@ -996,7 +989,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * \see strokeWidthUnit()
      * \see setStrokeWidth()
      * \see setStrokeWidthMapUnitScale()
-     * \since QGIS 2.16
     */
     void setStrokeWidthUnit( Qgis::RenderUnit unit ) { mStrokeWidthUnit = unit; }
 
@@ -1007,7 +999,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * \see strokeWidth()
      * \see strokeWidthUnit()
      *
-     * \since QGIS 2.16
     */
     const QgsMapUnitScale &strokeWidthMapUnitScale() const { return mStrokeWidthMapUnitScale; }
 
@@ -1018,7 +1009,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      * \see setStrokeWidth()
      * \see setStrokeWidthUnit()
      *
-     * \since QGIS 2.16
      */
     void setStrokeWidthMapUnitScale( const QgsMapUnitScale &scale ) { mStrokeWidthMapUnitScale = scale; }
 
@@ -1027,7 +1017,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      *
      * \see setPenJoinStyle()
      *
-     * \since QGIS 2.16
     */
     Qt::PenJoinStyle penJoinStyle() const { return mPenJoinStyle; }
 
@@ -1036,7 +1025,6 @@ class CORE_EXPORT QgsFontMarkerSymbolLayer : public QgsMarkerSymbolLayer
      *
      * \see penJoinStyle()
      *
-     * \since QGIS 2.16
     */
     void setPenJoinStyle( Qt::PenJoinStyle style ) { mPenJoinStyle = style; }
 
@@ -1106,7 +1094,7 @@ class CORE_EXPORT QgsAnimatedMarkerSymbolLayer : public QgsRasterMarkerSymbolLay
     /**
      * Creates an animated marker symbol layer from a string map of \a properties.
      */
-    static QgsSymbolLayer *create( const QVariantMap &properties = QVariantMap() ) SIP_FACTORY;
+    static QgsSymbolLayer *create( const QVariantMap &properties = QVariantMap() ) SIP_FACTORY; // cppcheck-suppress duplInheritedMember
 
     // implemented from base classes
 
